;**********************************************************
;                                                         ;
; File : MACR.INC                                         ;
; Basic Macros for programming TASM (MASM mode)           ;
; (C) 1994 Musa Maharramov                                ;
;                                                         ;
;**********************************************************


; the segment address of the video buffer
SCREEN_SEG	EQU	0B800H
MONO_SEG	EQU	0B000H

; eqauates for read/write and seek macros
_READ_WRITE	EQU	2
_READ_ONLY	EQU	1
_WRITE_ONLY	EQU	0

_START		EQU	0
_CURRENT	EQU	1
_END		EQU	2

; INT 24h return codes
_IGNORE		EQU	0
_RETRY		EQU	1
_ABORT		EQU	2

; keyboard codes (Scan/Ascii)
KeyLeft		EQU	4B00h
KeyUp		EQU	4800h
KeyRight	EQU	4D00h
KeyDown		EQU	5000h
KeyEnter	EQU	1C0Dh
KeyEsc		EQU	011Bh
KeyDel		EQU	5300h
KeyBackspace	EQU	0E08h
KeyF1		EQU	3B00h
KeyF2		EQU	3C00h
KeyF3		EQU	3D00h
KeyF4		EQU	3E00h
KeyF5		EQU	3F00h
KeyF6		EQU	4000h
KeyF7		EQU	4100h
KeyF8		EQU	4200h
KeyF9		EQU	4300h
KeyF10		EQU	4400h
KeyPgUp		EQU	4900h
KeyPgDn		EQU	5100h
KeyHome		EQU	4700h
KeyEnd		EQU	4F00h
CntrlHome	EQU	7700h
CntrlEnd	EQU	7500h
KeyTab		EQU	0F09h

KeyShF1		EQU	5400h
KeyShF2		EQU	5500h
KeyShF3		EQU	5600h
KeyShF4		EQU	5700h
KeyShF5		EQU	5800h
KeyShF6		EQU	5900h
KeyShF7		EQU	5A00h
KeyShF8		EQU	5B00h
KeyShF9		EQU	5C00h
KeyShF10	EQU	5D00h

; interrupt addresses
DOS_BREAK	EQU	23h
DOS_CRITERR	EQU	24h
BIOS_BREAK	EQU	1Bh
USER_TIMER	EQU	1Ch
TIMER		EQU	08h

; determines wether to display file I/O error messages
SHOW_ERR = 1	

	DATASEG
IOERR	DB	'File I/O error occured:0',10,13,'$'
Len	EQU	$-IOERR

	CODESEG
; FIOErr is called upon whenever an I/O error occured
; displays a message along with the error code
; Preserves the flags and AX
FIOErr	PROC	NEAR
IF	SHOW_ERR EQ 1	;; displays the message if SHOW_ERR is 1
	push	ax
	pushf
	add	IOERR[Len-4],al
	lea	dx,IOERR
	mov	ah,09h
	int	21h
	mov	IOERR[Len-4],'0'
	popf
	pop	ax
ENDIF
	ret
FIOErr	ENDP


; Moves the contents of arg2 into arg1 if arg1<>arg2
move	MACRO	arg1,arg2
IFDIFI	<arg1>,<arg2>
	mov	arg1,arg2
ENDIF
ENDM

; Genearates an error if arg1 and arg2 happen to coincide
ErrMatch MACRO arg1,arg2
IFIDNI <arg1>,<arg2>
     .ERR &arg1 mustn't be here
ENDIF
ENDM

; Shows a string on the standart output device
; The string is stored at Seg:Ofs
; Defaults for Seg:Ofs are DX:DS
ShowStr	MACRO	Ofs:=<dx>,Segm
IFNB	<Segm>
	push	ds
	move	ax,<Segm>
	mov	ds,ax
ENDIF
ErrMatch	<Ofs>,<ax>	;; must be in angle brackets  
	move	dx,<Ofs>	;; to avoid muddling up in case 
	mov	ah,09h		;; of compound expressions
	int	21h
IFNB	<Segm>
	pop	ds
ENDIF	
ENDM


; Sets the cursor to the position X:Y
; Preserves all the registers
SetCursor	MACRO X:=<dl>,Y:=<dh>
	pusha
ErrMatch	<X>,dh
ErrMatch	<Y>,dl
	move	dh,Y
	move	dl,X
	mov	ah,02h
	xor	bx,bx
	int	10h
	popa
ENDM

; Skips <Marg> characters up to and including Mark
; Seg:=<es>,Ofs:=<di>,Margin:=<cx>
; Destroys ax,cx,di,flags
SKIP	MACRO Marg,Ofs,Seg,Mrk:=<13>
IFNB	<Seg>
	push	es
	move	ax,<Seg>
	mov	es,ax
ENDIF

IFNB	<Ofs>
	move	di,<Ofs>
ENDIF

IFNB	<Marg>
ErrMatch	<ax>,<Marg>
	move	cx,<Marg>
ENDIF
	mov	al,Mrk
	cld
repne	scasb

IFNB	<Seg>
	pop	es
ENDIF

ENDM

; Opens a file
; FName:=<ds:dx>
; Default mode = READ/WRITE (2)
Open	MACRO	FName,Mode:=<2>
LOCAL	FNm,Ok,Err
IFNB	<FName>
	.DATA
FNm	DB	'&FName',0
	.CODE
	mov	dx,offset FNm
ENDIF
	move	al,Mode
	mov	ah,3Dh
	int	21h

	jnc	Ok
	call	FIOErr
Ok:

ENDM

; Creates a file
; FName:=<ds:dx>
; Default attribute = Normal (0)
Create	MACRO	FName,Mode:=<0>
LOCAL	FNm,Ok,Err
IFNB	<FName>
	.DATA
FNm	DB	'&FName',0
	.CODE
	mov	dx,offset FNm
ENDIF
	move	al,<Mode>
	mov	ah,3Ch
	move	cx,<Mode>
	int	21h

	jnc	Ok
	call	FIOErr
Ok:
ENDM

; Closes a file handle 
CLOSE	MACRO	Handle
IFNB	<Handle>
	move	bx,<Handle>
ENDIF
	mov	ah,3Eh
	int	21h
ENDM

	CODESEG

ReadBlock	PROC	PASCAL	NEAR
	DATASEG
RdB	DW	?
	CODESEG
ARG	Hndle:WORD,Cnt:WORD,BuffAddr:DWORD
USES	ds
	lds	dx,BuffAddr
	mov	bx,Hndle
	mov	cx,Cnt
	mov	ah,3Fh
	int	21h
	mov	RdB,ax
	jnc	OkR
	call	FIOErr
OkR:
	ret
ReadBlock	ENDP

; Reads Count Bytes from a file
; BuffOfs:=<dx>,BuffSeg:=<ds>
Read	MACRO	Handle:=<0>,Count:=<cx>,BuffOfs,BuffSeg
LOCAL	Ok
IFNB	<BuffSeg>
	push	ds
	move	ax,<BuffSeg>
	mov	ds,ax
ENDIF

IFNB	<BuffOfs>
ErrMatch	<BuffOfs>,ax
	move	dx,<BuffOfs>
ENDIF

ErrMatch	<Handle>,<ax>
ErrMatch	<Count>,<ax>
	move	bx,<Handle>
	move	cx,<Count>

	mov	ah,3Fh
	int	21h
	jnc	Ok
	call	FIOErr
Ok:
IFNB	<BuffSeg>
	pop	ds
ENDIF
ENDM

	CODESEG

WriteBlock	PROC	PASCAL	NEAR
ARG	Hndl:WORD,Cn:WORD,BuffAd:DWORD
USES	ds
	lds	dx,BuffAd
	mov	bx,Hndl
	mov	cx,Cn
	mov	ah,40h
	int	21h
	jnc	OkWr
	call	FIOErr
OkWr:
	ret
WriteBlock	ENDP


Write	MACRO	Handle:=<0>,Count:REQ,BuffOfs,BuffSeg
LOCAL	Ok
IFNB	<BuffSeg>
	push	ds
	move	ax,<BuffSeg>
	mov	ds,ax
ENDIF
IFNB	<BuffOfs>
        ErrMatch <BuffOfs>,ax
        move	dx,<BuffOfs>
ENDIF

ErrMatch	<Handle>,ax
ErrMatch	<Count>,ax
	move	bx,<Handle>
	move	cx,<Count>

	mov	ah,40h
	int	21h
	jnc	Ok
	call	FIOErr
Ok:
IFNB	<BuffSeg>
	pop	ds
ENDIF

ENDM


; Short Seek - places a file pointer at the position 00:Ofs
ShortSeek	MACRO	Handle,ofs,mode:REQ
	mov	ah,42h
IFNB	<Handle>
	move	bx,<Handle>
ENDIF
	xor	cx,cx
IFNB	<Ofs>
	move	dx,<Ofs>
ENDIF
	move	al,<mode>
	int	21h
ENDM

; Dos Buffered Input
; Seg:Ofs provides the address of a buffer
InputStr	MACRO	Ofs,Seg
IFNB	<Seg>
	push	ds
	move	ax,<Seg>
	mov	ds,ax
ENDIF
	mov	ah,0Ah
ErrMatch	<Ofs>,<ax>
	move	dx,<Ofs>
	int	21h
IFNB	<Seg>
	pop	ds
ENDIF

ENDM

; truncate a file
Truncate	MACRO	Handle
IFNB	<Handle>
	move	bx,<Handle>
ENDIF
	mov	ah,40h
	xor	cx,cx
	int	21h
ENDM

; Buffered input of an ASCIIZ string
; Instead of the 0 there might be any delimeter 
InputStrZ	MACRO	Ofs:REQ,Seg,Delim:=<0>
IFNB	<Seg>
	push	ds
	move	ax,<Seg>
	mov	ds,ax
ENDIF
	mov	ah,0Ah
ErrMatch	<Ofs>,<ax>
	move	dx,<Ofs>
	int	21h

	push	bx
	mov	bx,dx
	xor	ax,ax
	inc	bx
	mov	al,[bx]
	inc	bx
	add	bx,ax
	mov	BYTE PTR [bx],Delim
	pop	bx

IFNB	<Seg>
	pop	ds
ENDIF

ENDM


; Scroll a specified window Num lines Up
; Fills it in with Attr attribute 
; Destroys AX,SP,DI,BP,DI
ScrollUp	MACRO	Num:=<0>,Attr:REQ,LRow,LColumn,RRow,RColumn
	move	al,Num
	move	bh,Attr
	
IFNB	<LRow>
	move	ch,LRow
ENDIF


IFNB	<LColumn>
	move	cl,LColumn
ENDIF
IFNB	<RRow>
	move	dh,RRow
ENDIF

IFNB	<RColumn>
	move	dl,RColumn
ENDIF

	mov	ah,06h
	int	10h
ENDM



; Scroll a specified window Num lines Down
; Fills it in with Attr attribute 
ScrollDown	MACRO	Num:=<0>,Attr:REQ,LRow,LColumn,RRow,RColumn
	move	al,Num
	move	bh,Attr
	
IFNB	<LRow>
	move	ch,LRow
ENDIF

IFNB	<LColumn>
	move	cl,LColumn
ENDIF

IFNB	<RRow>
	move	dh,RRow
ENDIF

IFNB	<RColumn>
	move	dl,RColumn
ENDIF
	mov	ah,07h
	int	10h
ENDM

; set/get an interrupt vector macros
SetVector	PROC	PASCAL	NEAR
ARG	IntNum:BYTE,PROCADDR:DWORD
USES	ds
	lds	dx,PROCADDR
	mov	al,IntNum
	mov	ah,25h
	int	21h
	ret
SetVector	ENDP

; set/get an interrupt vector macros
SetVec	MACRO	IntNum:REQ,ofs:REQ,seg:REQ
	push	ds
	move	dx,<Ofs>
	move	ax,<Seg>
	mov	ds,ax
	mov	al,IntNum
	mov	ah,25h
	int	21h
	pop	ds
ENDM

GetVec	MACRO	IntNum:REQ
	mov	ah,35h
	move	al,<Intnum>
	int	21h
ENDM
